import expect from 'expect';
import elementRoleMap from '../../src/elementRoleMap';

describe('elementRoleMap API', function () {
  it('entries', function () {
    expect(elementRoleMap.entries()).toEqual(
      expect.arrayContaining(
        [
          [{"name": "menuitem"}, ["command", "menuitem"]],
          [{"name": "rel"}, ["roletype"]],
          [{"name": "article"}, ["article"]],
          [{"constraints": ["direct descendant of document"], "name": "header"}, ["banner"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-pressed"}, {"name": "type", "value": "checkbox"}], "name": "input"}, ["button"]],
          [{"attributes": [{"name": "aria-expanded", "value": "false"}], "name": "summary"}, ["button"]],
          [{"attributes": [{"name": "aria-expanded", "value": "true"}], "constraints": ["direct descendant of details element with the open attribute defined"], "name": "summary"}, ["button"]],
          [{"attributes": [{"name": "type", "value": "button"}], "name": "input"}, ["button"]],
          [{"attributes": [{"name": "type", "value": "image"}], "name": "input"}, ["button"]],
          [{"attributes": [{"name": "type", "value": "reset"}], "name": "input"}, ["button"]],
          [{"attributes": [{"name": "type", "value": "submit"}], "name": "input"}, ["button"]],
          [{"name": "button"}, ["button"]],
          [{"constraints": ["descendant of table"], "name": "td"}, ["cell"]],
          [{"attributes": [{"name": "type", "value": "checkbox"}], "name": "input"}, ["checkbox"]],
          [{"name": "th"}, ["columnheader"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "email"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "search"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "tel"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "text"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "multiple"}, {"constraints": ["undefined"], "name": "size"}], "name": "select"}, ["combobox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "multiple"}, {"name": "size", "value": 1}], "name": "select"}, ["combobox"]],
          [{"name": "aside"}, ["complementary"]],
          [{"constraints": ["direct descendant of document"], "name": "footer"}, ["contentinfo"]],
          [{"name": "dd"}, ["definition"]],
          [{"name": "dialog"}, ["dialog"]],
          [{"name": "body"}, ["document"]],
          [{"name": "figure"}, ["figure"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-label"}], "name": "form"}, ["form"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-labelledby"}], "name": "form"}, ["form"]],
          [{"attributes": [{"constraints": ["set"], "name": "name"}], "name": "form"}, ["form"]],
          [{"name": "span"}, ["generic"]],
          [{"name": "div"}, ["generic"]],
          [{"attributes": [{"name": "role", "value": "grid"}], "name": "table"}, ["grid"]],
          [{"attributes": [{"name": "role", "value": "gridcell"}], "name": "td"}, ["gridcell"]],
          [{"name": "details"}, ["group"]],
          [{"name": "fieldset"}, ["group"]],
          [{"name": "optgroup"}, ["group"]],
          [{"name": "h1"}, ["heading"]],
          [{"name": "h2"}, ["heading"]],
          [{"name": "h3"}, ["heading"]],
          [{"name": "h4"}, ["heading"]],
          [{"name": "h5"}, ["heading"]],
          [{"name": "h6"}, ["heading"]],
          [{"attributes": [{"constraints": ["set"], "name": "alt"}], "name": "img"}, ["img"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "alt"}], "name": "img"}, ["img"]],
          [{"attributes": [{"name": "href"}], "name": "a"}, ["link"]],
          [{"attributes": [{"name": "href"}], "name": "area"}, ["link"]],
          [{"attributes": [{"name": "href"}], "name": "link"}, ["link"]],
          [{"name": "menu"}, ["list"]],
          [{"name": "ol"}, ["list"]],
          [{"name": "ul"}, ["list"]],
          [{"attributes": [{"constraints": [">1"], "name": "size"}, {"name": "multiple"}], "name": "select"}, ["listbox"]],
          [{"attributes": [{"constraints": [">1"], "name": "size"}], "name": "select"}, ["listbox"]],
          [{"attributes": [{"name": "multiple"}], "name": "select"}, ["listbox"]],
          [{"name": "datalist"}, ["listbox"]],
          [{"constraints": ["direct descendant of ol, ul or menu"], "name": "li"}, ["listitem"]],
          [{"name": "main"}, ["main"]],
          [{"name": "math"}, ["math"]],
          [{"name": "menuitem"}, ["command", "menuitem"]],
          [{"name": "nav"}, ["navigation"]],
          [{"name": "option"}, ["option"]],
          [{"name": "progress"}, ["progressbar"]],
          [{"attributes": [{"name": "type", "value": "radio"}], "name": "input"}, ["radio"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-label"}], "name": "section"}, ["region"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-labelledby"}], "name": "section"}, ["region"]],
          [{"name": "frame"}, ["region"]],
          [{"name": "tr"}, ["row"]],
          [{"name": "tbody"}, ["rowgroup"]],
          [{"name": "tfoot"}, ["rowgroup"]],
          [{"name": "thead"}, ["rowgroup"]],
          [{"attributes": [{"name": "scope", "value": "row"}], "name": "th"}, ["rowheader"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "search"}], "name": "input"}, ["searchbox"]],
          [{"name": "hr"}, ["separator"]],
          [{"attributes": [{"name": "type", "value": "range"}], "name": "input"}, ["slider"]],
          [{"attributes": [{"name": "type", "value": "number"}], "name": "input"}, ["spinbutton"]],
          [{"name": "output"}, ["status"]],
          [{"name": "table"}, ["table"]],
          [{"name": "dfn"}, ["term"]],
          [{"name": "dt"}, ["term"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "type"}, {"constraints": ["undefined"], "name": "list"}], "name": "input"}, ["textbox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "email"}], "name": "input"}, ["textbox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "tel"}], "name": "input"}, ["textbox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "text"}], "name": "input"}, ["textbox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["textbox"]],
          [{"name": "textarea"}, ["textbox"]]
        ]
      )
    );
    expect([...elementRoleMap.entries()]).toEqual(
      expect.arrayContaining(
        [
          [{"name": "menuitem"}, ["command", "menuitem"]],
          [{"name": "rel"}, ["roletype"]],
          [{"name": "article"}, ["article"]],
          [{"constraints": ["direct descendant of document"], "name": "header"}, ["banner"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-pressed"}, {"name": "type", "value": "checkbox"}], "name": "input"}, ["button"]],
          [{"attributes": [{"name": "aria-expanded", "value": "false"}], "name": "summary"}, ["button"]],
          [{"attributes": [{"name": "aria-expanded", "value": "true"}], "constraints": ["direct descendant of details element with the open attribute defined"], "name": "summary"}, ["button"]],
          [{"attributes": [{"name": "type", "value": "button"}], "name": "input"}, ["button"]],
          [{"attributes": [{"name": "type", "value": "image"}], "name": "input"}, ["button"]],
          [{"attributes": [{"name": "type", "value": "reset"}], "name": "input"}, ["button"]],
          [{"attributes": [{"name": "type", "value": "submit"}], "name": "input"}, ["button"]],
          [{"name": "button"}, ["button"]],
          [{"constraints": ["descendant of table"], "name": "td"}, ["cell"]],
          [{"attributes": [{"name": "type", "value": "checkbox"}], "name": "input"}, ["checkbox"]],
          [{"name": "th"}, ["columnheader"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "email"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "search"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "tel"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "text"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "multiple"}, {"constraints": ["undefined"], "name": "size"}], "name": "select"}, ["combobox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "multiple"}, {"name": "size", "value": 1}], "name": "select"}, ["combobox"]],
          [{"name": "aside"}, ["complementary"]],
          [{"constraints": ["direct descendant of document"], "name": "footer"}, ["contentinfo"]],
          [{"name": "dd"}, ["definition"]],
          [{"name": "dialog"}, ["dialog"]],
          [{"name": "body"}, ["document"]],
          [{"name": "figure"}, ["figure"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-label"}], "name": "form"}, ["form"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-labelledby"}], "name": "form"}, ["form"]],
          [{"attributes": [{"constraints": ["set"], "name": "name"}], "name": "form"}, ["form"]],
          [{"name": "span"}, ["generic"]],
          [{"name": "div"}, ["generic"]],
          [{"attributes": [{"name": "role", "value": "grid"}], "name": "table"}, ["grid"]],
          [{"attributes": [{"name": "role", "value": "gridcell"}], "name": "td"}, ["gridcell"]],
          [{"name": "details"}, ["group"]],
          [{"name": "fieldset"}, ["group"]],
          [{"name": "optgroup"}, ["group"]],
          [{"name": "h1"}, ["heading"]],
          [{"name": "h2"}, ["heading"]],
          [{"name": "h3"}, ["heading"]],
          [{"name": "h4"}, ["heading"]],
          [{"name": "h5"}, ["heading"]],
          [{"name": "h6"}, ["heading"]],
          [{"attributes": [{"constraints": ["set"], "name": "alt"}], "name": "img"}, ["img"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "alt"}], "name": "img"}, ["img"]],
          [{"attributes": [{"name": "href"}], "name": "a"}, ["link"]],
          [{"attributes": [{"name": "href"}], "name": "area"}, ["link"]],
          [{"attributes": [{"name": "href"}], "name": "link"}, ["link"]],
          [{"name": "menu"}, ["list"]],
          [{"name": "ol"}, ["list"]],
          [{"name": "ul"}, ["list"]],
          [{"attributes": [{"constraints": [">1"], "name": "size"}, {"name": "multiple"}], "name": "select"}, ["listbox"]],
          [{"attributes": [{"constraints": [">1"], "name": "size"}], "name": "select"}, ["listbox"]],
          [{"attributes": [{"name": "multiple"}], "name": "select"}, ["listbox"]],
          [{"name": "datalist"}, ["listbox"]],
          [{"constraints": ["direct descendant of ol, ul or menu"], "name": "li"}, ["listitem"]],
          [{"name": "main"}, ["main"]],
          [{"name": "math"}, ["math"]],
          [{"name": "menuitem"}, ["command", "menuitem"]],
          [{"name": "nav"}, ["navigation"]],
          [{"name": "option"}, ["option"]],
          [{"name": "progress"}, ["progressbar"]],
          [{"attributes": [{"name": "type", "value": "radio"}], "name": "input"}, ["radio"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-label"}], "name": "section"}, ["region"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-labelledby"}], "name": "section"}, ["region"]],
          [{"name": "frame"}, ["region"]],
          [{"name": "tr"}, ["row"]],
          [{"name": "tbody"}, ["rowgroup"]],
          [{"name": "tfoot"}, ["rowgroup"]],
          [{"name": "thead"}, ["rowgroup"]],
          [{"attributes": [{"name": "scope", "value": "row"}], "name": "th"}, ["rowheader"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "search"}], "name": "input"}, ["searchbox"]],
          [{"name": "hr"}, ["separator"]],
          [{"attributes": [{"name": "type", "value": "range"}], "name": "input"}, ["slider"]],
          [{"attributes": [{"name": "type", "value": "number"}], "name": "input"}, ["spinbutton"]],
          [{"name": "output"}, ["status"]],
          [{"name": "table"}, ["table"]],
          [{"name": "dfn"}, ["term"]],
          [{"name": "dt"}, ["term"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "type"}, {"constraints": ["undefined"], "name": "list"}], "name": "input"}, ["textbox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "email"}], "name": "input"}, ["textbox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "tel"}], "name": "input"}, ["textbox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "text"}], "name": "input"}, ["textbox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["textbox"]],
          [{"name": "textarea"}, ["textbox"]]
        ]
      )
    );
  });
  it('forEach', function () {
    const output = [];
    let context;
    elementRoleMap.forEach((value, key, map) => {
      output.push([key, value]);
      if (!context) {
        context = map;
      }
    });
    expect(output).toEqual(
      expect.arrayContaining(
        [
          [{"name": "menuitem"}, ["command", "menuitem"]],
          [{"name": "rel"}, ["roletype"]],
          [{"name": "article"}, ["article"]],
          [{"constraints": ["direct descendant of document"], "name": "header"}, ["banner"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-pressed"}, {"name": "type", "value": "checkbox"}], "name": "input"}, ["button"]],
          [{"attributes": [{"name": "aria-expanded", "value": "false"}], "name": "summary"}, ["button"]],
          [{"attributes": [{"name": "aria-expanded", "value": "true"}], "constraints": ["direct descendant of details element with the open attribute defined"], "name": "summary"}, ["button"]],
          [{"attributes": [{"name": "type", "value": "button"}], "name": "input"}, ["button"]],
          [{"attributes": [{"name": "type", "value": "image"}], "name": "input"}, ["button"]],
          [{"attributes": [{"name": "type", "value": "reset"}], "name": "input"}, ["button"]],
          [{"attributes": [{"name": "type", "value": "submit"}], "name": "input"}, ["button"]],
          [{"name": "button"}, ["button"]],
          [{"constraints": ["descendant of table"], "name": "td"}, ["cell"]],
          [{"attributes": [{"name": "type", "value": "checkbox"}], "name": "input"}, ["checkbox"]],
          [{"name": "th"}, ["columnheader"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "email"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "search"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "tel"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "text"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "multiple"}, {"constraints": ["undefined"], "name": "size"}], "name": "select"}, ["combobox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "multiple"}, {"name": "size", "value": 1}], "name": "select"}, ["combobox"]],
          [{"name": "aside"}, ["complementary"]],
          [{"constraints": ["direct descendant of document"], "name": "footer"}, ["contentinfo"]],
          [{"name": "dd"}, ["definition"]],
          [{"name": "dialog"}, ["dialog"]],
          [{"name": "body"}, ["document"]],
          [{"name": "figure"}, ["figure"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-label"}], "name": "form"}, ["form"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-labelledby"}], "name": "form"}, ["form"]],
          [{"attributes": [{"constraints": ["set"], "name": "name"}], "name": "form"}, ["form"]],
          [{"name": "span"}, ["generic"]],
          [{"name": "div"}, ["generic"]],
          [{"attributes": [{"name": "role", "value": "grid"}], "name": "table"}, ["grid"]],
          [{"attributes": [{"name": "role", "value": "gridcell"}], "name": "td"}, ["gridcell"]],
          [{"name": "details"}, ["group"]],
          [{"name": "fieldset"}, ["group"]],
          [{"name": "optgroup"}, ["group"]],
          [{"name": "h1"}, ["heading"]],
          [{"name": "h2"}, ["heading"]],
          [{"name": "h3"}, ["heading"]],
          [{"name": "h4"}, ["heading"]],
          [{"name": "h5"}, ["heading"]],
          [{"name": "h6"}, ["heading"]],
          [{"attributes": [{"constraints": ["set"], "name": "alt"}], "name": "img"}, ["img"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "alt"}], "name": "img"}, ["img"]],
          [{"attributes": [{"name": "href"}], "name": "a"}, ["link"]],
          [{"attributes": [{"name": "href"}], "name": "area"}, ["link"]],
          [{"attributes": [{"name": "href"}], "name": "link"}, ["link"]],
          [{"name": "menu"}, ["list"]],
          [{"name": "ol"}, ["list"]],
          [{"name": "ul"}, ["list"]],
          [{"attributes": [{"constraints": [">1"], "name": "size"}, {"name": "multiple"}], "name": "select"}, ["listbox"]],
          [{"attributes": [{"constraints": [">1"], "name": "size"}], "name": "select"}, ["listbox"]],
          [{"attributes": [{"name": "multiple"}], "name": "select"}, ["listbox"]],
          [{"name": "datalist"}, ["listbox"]],
          [{"constraints": ["direct descendant of ol, ul or menu"], "name": "li"}, ["listitem"]],
          [{"name": "main"}, ["main"]],
          [{"name": "math"}, ["math"]],
          [{"name": "menuitem"}, ["command", "menuitem"]],
          [{"name": "nav"}, ["navigation"]],
          [{"name": "option"}, ["option"]],
          [{"name": "progress"}, ["progressbar"]],
          [{"attributes": [{"name": "type", "value": "radio"}], "name": "input"}, ["radio"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-label"}], "name": "section"}, ["region"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-labelledby"}], "name": "section"}, ["region"]],
          [{"name": "frame"}, ["region"]],
          [{"name": "tr"}, ["row"]],
          [{"name": "tbody"}, ["rowgroup"]],
          [{"name": "tfoot"}, ["rowgroup"]],
          [{"name": "thead"}, ["rowgroup"]],
          [{"attributes": [{"name": "scope", "value": "row"}], "name": "th"}, ["rowheader"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "search"}], "name": "input"}, ["searchbox"]],
          [{"name": "hr"}, ["separator"]],
          [{"attributes": [{"name": "type", "value": "range"}], "name": "input"}, ["slider"]],
          [{"attributes": [{"name": "type", "value": "number"}], "name": "input"}, ["spinbutton"]],
          [{"name": "output"}, ["status"]],
          [{"name": "table"}, ["table"]],
          [{"name": "dfn"}, ["term"]],
          [{"name": "dt"}, ["term"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "type"}, {"constraints": ["undefined"], "name": "list"}], "name": "input"}, ["textbox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "email"}], "name": "input"}, ["textbox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "tel"}], "name": "input"}, ["textbox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "text"}], "name": "input"}, ["textbox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["textbox"]],
          [{"name": "textarea"}, ["textbox"]]
        ]
      )
    );
    expect(context).toEqual(
      expect.arrayContaining(
        [
          [{"name": "menuitem"}, ["command", "menuitem"]],
          [{"name": "rel"}, ["roletype"]],
          [{"name": "article"}, ["article"]],
          [{"constraints": ["direct descendant of document"], "name": "header"}, ["banner"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-pressed"}, {"name": "type", "value": "checkbox"}], "name": "input"}, ["button"]],
          [{"attributes": [{"name": "aria-expanded", "value": "false"}], "name": "summary"}, ["button"]],
          [{"attributes": [{"name": "aria-expanded", "value": "true"}], "constraints": ["direct descendant of details element with the open attribute defined"], "name": "summary"}, ["button"]],
          [{"attributes": [{"name": "type", "value": "button"}], "name": "input"}, ["button"]],
          [{"attributes": [{"name": "type", "value": "image"}], "name": "input"}, ["button"]],
          [{"attributes": [{"name": "type", "value": "reset"}], "name": "input"}, ["button"]],
          [{"attributes": [{"name": "type", "value": "submit"}], "name": "input"}, ["button"]],
          [{"name": "button"}, ["button"]],
          [{"constraints": ["descendant of table"], "name": "td"}, ["cell"]],
          [{"attributes": [{"name": "type", "value": "checkbox"}], "name": "input"}, ["checkbox"]],
          [{"name": "th"}, ["columnheader"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "email"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "search"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "tel"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "text"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["combobox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "multiple"}, {"constraints": ["undefined"], "name": "size"}], "name": "select"}, ["combobox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "multiple"}, {"name": "size", "value": 1}], "name": "select"}, ["combobox"]],
          [{"name": "aside"}, ["complementary"]],
          [{"constraints": ["direct descendant of document"], "name": "footer"}, ["contentinfo"]],
          [{"name": "dd"}, ["definition"]],
          [{"name": "dialog"}, ["dialog"]],
          [{"name": "body"}, ["document"]],
          [{"name": "figure"}, ["figure"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-label"}], "name": "form"}, ["form"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-labelledby"}], "name": "form"}, ["form"]],
          [{"attributes": [{"constraints": ["set"], "name": "name"}], "name": "form"}, ["form"]],
          [{"name": "span"}, ["generic"]],
          [{"name": "div"}, ["generic"]],
          [{"attributes": [{"name": "role", "value": "grid"}], "name": "table"}, ["grid"]],
          [{"attributes": [{"name": "role", "value": "gridcell"}], "name": "td"}, ["gridcell"]],
          [{"name": "details"}, ["group"]],
          [{"name": "fieldset"}, ["group"]],
          [{"name": "optgroup"}, ["group"]],
          [{"name": "h1"}, ["heading"]],
          [{"name": "h2"}, ["heading"]],
          [{"name": "h3"}, ["heading"]],
          [{"name": "h4"}, ["heading"]],
          [{"name": "h5"}, ["heading"]],
          [{"name": "h6"}, ["heading"]],
          [{"attributes": [{"constraints": ["set"], "name": "alt"}], "name": "img"}, ["img"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "alt"}], "name": "img"}, ["img"]],
          [{"attributes": [{"name": "href"}], "name": "a"}, ["link"]],
          [{"attributes": [{"name": "href"}], "name": "area"}, ["link"]],
          [{"attributes": [{"name": "href"}], "name": "link"}, ["link"]],
          [{"name": "menu"}, ["list"]],
          [{"name": "ol"}, ["list"]],
          [{"name": "ul"}, ["list"]],
          [{"attributes": [{"constraints": [">1"], "name": "size"}, {"name": "multiple"}], "name": "select"}, ["listbox"]],
          [{"attributes": [{"constraints": [">1"], "name": "size"}], "name": "select"}, ["listbox"]],
          [{"attributes": [{"name": "multiple"}], "name": "select"}, ["listbox"]],
          [{"name": "datalist"}, ["listbox"]],
          [{"constraints": ["direct descendant of ol, ul or menu"], "name": "li"}, ["listitem"]],
          [{"name": "main"}, ["main"]],
          [{"name": "math"}, ["math"]],
          [{"name": "menuitem"}, ["command", "menuitem"]],
          [{"name": "nav"}, ["navigation"]],
          [{"name": "option"}, ["option"]],
          [{"name": "progress"}, ["progressbar"]],
          [{"attributes": [{"name": "type", "value": "radio"}], "name": "input"}, ["radio"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-label"}], "name": "section"}, ["region"]],
          [{"attributes": [{"constraints": ["set"], "name": "aria-labelledby"}], "name": "section"}, ["region"]],
          [{"name": "frame"}, ["region"]],
          [{"name": "tr"}, ["row"]],
          [{"name": "tbody"}, ["rowgroup"]],
          [{"name": "tfoot"}, ["rowgroup"]],
          [{"name": "thead"}, ["rowgroup"]],
          [{"attributes": [{"name": "scope", "value": "row"}], "name": "th"}, ["rowheader"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "search"}], "name": "input"}, ["searchbox"]],
          [{"name": "hr"}, ["separator"]],
          [{"attributes": [{"name": "type", "value": "range"}], "name": "input"}, ["slider"]],
          [{"attributes": [{"name": "type", "value": "number"}], "name": "input"}, ["spinbutton"]],
          [{"name": "output"}, ["status"]],
          [{"name": "table"}, ["table"]],
          [{"name": "dfn"}, ["term"]],
          [{"name": "dt"}, ["term"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "type"}, {"constraints": ["undefined"], "name": "list"}], "name": "input"}, ["textbox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "email"}], "name": "input"}, ["textbox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "tel"}], "name": "input"}, ["textbox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "text"}], "name": "input"}, ["textbox"]],
          [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["textbox"]],
          [{"name": "textarea"}, ["textbox"]]
        ]
      )
    );
  });
  it('get', function () {
    expect(elementRoleMap.get({
      attributes: [
        { name: 'href' }
      ],
      name: 'a'
    })).toEqual(
      expect.arrayContaining(["link"]),
    );
    expect(elementRoleMap.get({
      "attributes": [
        {
          "name": "type",
          "value": "radio"
        }
      ], "name": "input"
    })).toEqual(
      expect.arrayContaining(["radio"]),
    );
    expect(elementRoleMap.get({
      attributes: {
        name: 'fake attribute',
      },
      name: 'fake element',
    })).toBeUndefined();
  });
  it('has', function () {
    expect(elementRoleMap.has({
      attributes: [
        { name: 'href' }
      ],
      name: 'a'
    })).toEqual(true);
    expect(elementRoleMap.has({
      attributes: {
        name: 'fake attribute',
      },
      name: 'fake element',
    })).toEqual(false);
  });
  it('keys', function () {
    expect(elementRoleMap.keys()).toEqual(
      expect.arrayContaining(
        [
          {"name": "menuitem"},
          {"name": "rel"},
          {"name": "article"},
          {"constraints": ["direct descendant of document"], "name": "header"},
          {"attributes": [{"constraints": ["set"], "name": "aria-pressed"}, {"name": "type", "value": "checkbox"}], "name": "input"},
          {"attributes": [{"name": "aria-expanded", "value": "false"}], "name": "summary"},
          {"attributes": [{"name": "aria-expanded", "value": "true"}], "constraints": ["direct descendant of details element with the open attribute defined"], "name": "summary"},
          {"attributes": [{"name": "type", "value": "button"}], "name": "input"},
          {"attributes": [{"name": "type", "value": "image"}], "name": "input"},
          {"attributes": [{"name": "type", "value": "reset"}], "name": "input"},
          {"attributes": [{"name": "type", "value": "submit"}], "name": "input"},
          {"name": "button"},
          {"constraints": ["descendant of table"], "name": "td"},
          {"attributes": [{"name": "type", "value": "checkbox"}], "name": "input"},
          {"name": "th"},
          {"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "email"}], "name": "input"},
          {"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "search"}], "name": "input"},
          {"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "tel"}], "name": "input"},
          {"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "text"}], "name": "input"},
          {"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"},
          {"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"},
          {"attributes": [{"constraints": ["undefined"], "name": "multiple"}, {"constraints": ["undefined"], "name": "size"}], "name": "select"},
          {"attributes": [{"constraints": ["undefined"], "name": "multiple"}, {"name": "size", "value": 1}], "name": "select"},
          {"name": "aside"},
          {"constraints": ["direct descendant of document"], "name": "footer"},
          {"name": "dd"},
          {"name": "dialog"},
          {"name": "body"},
          {"name": "figure"},
          {"attributes": [{"constraints": ["set"], "name": "aria-label"}], "name": "form"},
          {"attributes": [{"constraints": ["set"], "name": "aria-labelledby"}], "name": "form"},
          {"attributes": [{"constraints": ["set"], "name": "name"}], "name": "form"},
          {"name": "span"},
          {"name": "div"},
          {"attributes": [{"name": "role", "value": "grid"}], "name": "table"},
          {"attributes": [{"name": "role", "value": "gridcell"}], "name": "td"},
          {"name": "details"},
          {"name": "fieldset"},
          {"name": "optgroup"},
          {"name": "h1"},
          {"name": "h2"},
          {"name": "h3"},
          {"name": "h4"},
          {"name": "h5"},
          {"name": "h6"},
          {"attributes": [{"constraints": ["set"], "name": "alt"}], "name": "img"},
          {"attributes": [{"constraints": ["undefined"], "name": "alt"}], "name": "img"},
          {"attributes": [{"name": "href"}], "name": "a"},
          {"attributes": [{"name": "href"}], "name": "area"},
          {"attributes": [{"name": "href"}], "name": "link"},
          {"name": "menu"},
          {"name": "ol"},
          {"name": "ul"},
          {"attributes": [{"constraints": [">1"], "name": "size"}, {"name": "multiple"}], "name": "select"},
          {"attributes": [{"constraints": [">1"], "name": "size"}], "name": "select"},
          {"attributes": [{"name": "multiple"}], "name": "select"},
          {"name": "datalist"},
          {"constraints": ["direct descendant of ol, ul or menu"], "name": "li"},
          {"name": "main"},
          {"name": "math"},
          {"name": "menuitem"},
          {"name": "nav"},
          {"name": "option"},
          {"name": "progress"},
          {"attributes": [{"name": "type", "value": "radio"}], "name": "input"},
          {"attributes": [{"constraints": ["set"], "name": "aria-label"}], "name": "section"},
          {"attributes": [{"constraints": ["set"], "name": "aria-labelledby"}], "name": "section"},
          {"name": "frame"},
          {"name": "tr"},
          {"name": "tbody"},
          {"name": "tfoot"},
          {"name": "thead"},
          {"attributes": [{"name": "scope", "value": "row"}], "name": "th"},
          {"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "search"}], "name": "input"},
          {"name": "hr"},
          {"attributes": [{"name": "type", "value": "range"}], "name": "input"},
          {"attributes": [{"name": "type", "value": "number"}], "name": "input"},
          {"name": "output"},
          {"name": "table"},
          {"name": "dfn"},
          {"name": "dt"},
          {"attributes": [{"constraints": ["undefined"], "name": "type"}, {"constraints": ["undefined"], "name": "list"}], "name": "input"},
          {"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "email"}], "name": "input"},
          {"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "tel"}], "name": "input"},
          {"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "text"}], "name": "input"},
          {"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"},
          {"name": "textarea"},
        ]
      )
    );
    expect([...elementRoleMap.keys()]).toEqual(
      expect.arrayContaining(
        [
          {"name": "menuitem"},
          {"name": "rel"},
          {"name": "article"},
          {"constraints": ["direct descendant of document"], "name": "header"},
          {"attributes": [{"constraints": ["set"], "name": "aria-pressed"}, {"name": "type", "value": "checkbox"}], "name": "input"},
          {"attributes": [{"name": "aria-expanded", "value": "false"}], "name": "summary"},
          {"attributes": [{"name": "aria-expanded", "value": "true"}], "constraints": ["direct descendant of details element with the open attribute defined"], "name": "summary"},
          {"attributes": [{"name": "type", "value": "button"}], "name": "input"},
          {"attributes": [{"name": "type", "value": "image"}], "name": "input"},
          {"attributes": [{"name": "type", "value": "reset"}], "name": "input"},
          {"attributes": [{"name": "type", "value": "submit"}], "name": "input"},
          {"name": "button"},
          {"constraints": ["descendant of table"], "name": "td"},
          {"attributes": [{"name": "type", "value": "checkbox"}], "name": "input"},
          {"name": "th"},
          {"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "email"}], "name": "input"},
          {"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "search"}], "name": "input"},
          {"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "tel"}], "name": "input"},
          {"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "text"}], "name": "input"},
          {"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"},
          {"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"},
          {"attributes": [{"constraints": ["undefined"], "name": "multiple"}, {"constraints": ["undefined"], "name": "size"}], "name": "select"},
          {"attributes": [{"constraints": ["undefined"], "name": "multiple"}, {"name": "size", "value": 1}], "name": "select"},
          {"name": "aside"},
          {"constraints": ["direct descendant of document"], "name": "footer"},
          {"name": "dd"},
          {"name": "dialog"},
          {"name": "body"},
          {"name": "figure"},
          {"attributes": [{"constraints": ["set"], "name": "aria-label"}], "name": "form"},
          {"attributes": [{"constraints": ["set"], "name": "aria-labelledby"}], "name": "form"},
          {"attributes": [{"constraints": ["set"], "name": "name"}], "name": "form"},
          {"name": "span"},
          {"name": "div"},
          {"attributes": [{"name": "role", "value": "grid"}], "name": "table"},
          {"attributes": [{"name": "role", "value": "gridcell"}], "name": "td"},
          {"name": "details"},
          {"name": "fieldset"},
          {"name": "optgroup"},
          {"name": "h1"},
          {"name": "h2"},
          {"name": "h3"},
          {"name": "h4"},
          {"name": "h5"},
          {"name": "h6"},
          {"attributes": [{"constraints": ["set"], "name": "alt"}], "name": "img"},
          {"attributes": [{"constraints": ["undefined"], "name": "alt"}], "name": "img"},
          {"attributes": [{"name": "href"}], "name": "a"},
          {"attributes": [{"name": "href"}], "name": "area"},
          {"attributes": [{"name": "href"}], "name": "link"},
          {"name": "menu"},
          {"name": "ol"},
          {"name": "ul"},
          {"attributes": [{"constraints": [">1"], "name": "size"}, {"name": "multiple"}], "name": "select"},
          {"attributes": [{"constraints": [">1"], "name": "size"}], "name": "select"},
          {"attributes": [{"name": "multiple"}], "name": "select"},
          {"name": "datalist"},
          {"constraints": ["direct descendant of ol, ul or menu"], "name": "li"},
          {"name": "main"},
          {"name": "math"},
          {"name": "menuitem"},
          {"name": "nav"},
          {"name": "option"},
          {"name": "progress"},
          {"attributes": [{"name": "type", "value": "radio"}], "name": "input"},
          {"attributes": [{"constraints": ["set"], "name": "aria-label"}], "name": "section"},
          {"attributes": [{"constraints": ["set"], "name": "aria-labelledby"}], "name": "section"},
          {"name": "frame"},
          {"name": "tr"},
          {"name": "tbody"},
          {"name": "tfoot"},
          {"name": "thead"},
          {"attributes": [{"name": "scope", "value": "row"}], "name": "th"},
          {"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "search"}], "name": "input"},
          {"name": "hr"},
          {"attributes": [{"name": "type", "value": "range"}], "name": "input"},
          {"attributes": [{"name": "type", "value": "number"}], "name": "input"},
          {"name": "output"},
          {"name": "table"},
          {"name": "dfn"},
          {"name": "dt"},
          {"attributes": [{"constraints": ["undefined"], "name": "type"}, {"constraints": ["undefined"], "name": "list"}], "name": "input"},
          {"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "email"}], "name": "input"},
          {"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "tel"}], "name": "input"},
          {"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "text"}], "name": "input"},
          {"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"},
          {"name": "textarea"},
        ]
      )
    );
  });
  it('values', function () {
    expect(elementRoleMap.values()).toEqual(
      expect.arrayContaining(
        [
          ["command", "menuitem"],
          ["roletype"],
          ["article"],
          ["banner"],
          ["button"],
          ["button"],
          ["button"],
          ["button"],
          ["button"],
          ["button"],
          ["button"],
          ["button"],
          ["cell"],
          ["checkbox"],
          ["columnheader"],
          ["combobox"],
          ["combobox"],
          ["combobox"],
          ["combobox"],
          ["combobox"],
          ["combobox"],
          ["combobox"],
          ["combobox"],
          ["complementary"],
          ["contentinfo"],
          ["definition"],
          ["dialog"],
          ["document"],
          ["figure"],
          ["form"],
          ["form"],
          ["form"],
          ["generic"],
          ["generic"],
          ["grid"],
          ["gridcell"],
          ["group"],
          ["group"],
          ["group"],
          ["heading"],
          ["heading"],
          ["heading"],
          ["heading"],
          ["heading"],
          ["heading"],
          ["img"],
          ["img"],
          ["link"],
          ["link"],
          ["link"],
          ["list"],
          ["list"],
          ["list"],
          ["listbox"],
          ["listbox"],
          ["listbox"],
          ["listbox"],
          ["listitem"],
          ["main"],
          ["math"],
          ["command", "menuitem"],
          ["navigation"],
          ["option"],
          ["progressbar"],
          ["radio"],
          ["region"],
          ["region"],
          ["region"],
          ["row"],
          ["rowgroup"],
          ["rowgroup"],
          ["rowgroup"],
          ["rowheader"],
          ["searchbox"],
          ["separator"],
          ["slider"],
          ["spinbutton"],
          ["status"],
          ["table"],
          ["term"],
          ["term"],
          ["textbox"],
          ["textbox"],
          ["textbox"],
          ["textbox"],
          ["textbox"],
          ["textbox"],
        ]
      )
    );
    expect([...elementRoleMap.values()]).toEqual(
      expect.arrayContaining(
        [
          ["command", "menuitem"],
          ["roletype"],
          ["article"],
          ["banner"],
          ["button"],
          ["button"],
          ["button"],
          ["button"],
          ["button"],
          ["button"],
          ["button"],
          ["button"],
          ["cell"],
          ["checkbox"],
          ["columnheader"],
          ["combobox"],
          ["combobox"],
          ["combobox"],
          ["combobox"],
          ["combobox"],
          ["combobox"],
          ["combobox"],
          ["combobox"],
          ["complementary"],
          ["contentinfo"],
          ["definition"],
          ["dialog"],
          ["document"],
          ["figure"],
          ["form"],
          ["form"],
          ["form"],
          ["generic"],
          ["generic"],
          ["grid"],
          ["gridcell"],
          ["group"],
          ["group"],
          ["group"],
          ["heading"],
          ["heading"],
          ["heading"],
          ["heading"],
          ["heading"],
          ["heading"],
          ["img"],
          ["img"],
          ["link"],
          ["link"],
          ["link"],
          ["list"],
          ["list"],
          ["list"],
          ["listbox"],
          ["listbox"],
          ["listbox"],
          ["listbox"],
          ["listitem"],
          ["main"],
          ["math"],
          ["command", "menuitem"],
          ["navigation"],
          ["option"],
          ["progressbar"],
          ["radio"],
          ["region"],
          ["region"],
          ["region"],
          ["row"],
          ["rowgroup"],
          ["rowgroup"],
          ["rowgroup"],
          ["rowheader"],
          ["searchbox"],
          ["separator"],
          ["slider"],
          ["spinbutton"],
          ["status"],
          ["table"],
          ["term"],
          ["term"],
          ["textbox"],
          ["textbox"],
          ["textbox"],
          ["textbox"],
          ["textbox"],
          ["textbox"],
        ]
      )
    );
  });
});

describe('elementRolesMap', function () {
  describe('iteration', function () {
    it('should have an iterator defined', function () {
      expect(elementRoleMap[Symbol.iterator]).not.toBeUndefined();
    });
    it('should support the spread operator', function () {
      expect([...elementRoleMap].length).toEqual(88);
      expect(elementRoleMap.entries()).toEqual(
        expect.arrayContaining(
          [
            [{"name": "menuitem"}, ["command", "menuitem"]],
            [{"name": "rel"}, ["roletype"]],
            [{"name": "article"}, ["article"]],
            [{"constraints": ["direct descendant of document"], "name": "header"}, ["banner"]],
            [{"attributes": [{"constraints": ["set"], "name": "aria-pressed"}, {"name": "type", "value": "checkbox"}], "name": "input"}, ["button"]],
            [{"attributes": [{"name": "aria-expanded", "value": "false"}], "name": "summary"}, ["button"]],
            [{"attributes": [{"name": "aria-expanded", "value": "true"}], "constraints": ["direct descendant of details element with the open attribute defined"], "name": "summary"}, ["button"]],
            [{"attributes": [{"name": "type", "value": "button"}], "name": "input"}, ["button"]],
            [{"attributes": [{"name": "type", "value": "image"}], "name": "input"}, ["button"]],
            [{"attributes": [{"name": "type", "value": "reset"}], "name": "input"}, ["button"]],
            [{"attributes": [{"name": "type", "value": "submit"}], "name": "input"}, ["button"]],
            [{"name": "button"}, ["button"]],
            [{"constraints": ["descendant of table"], "name": "td"}, ["cell"]],
            [{"attributes": [{"name": "type", "value": "checkbox"}], "name": "input"}, ["checkbox"]],
            [{"name": "th"}, ["columnheader"]],
            [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "email"}], "name": "input"}, ["combobox"]],
            [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "search"}], "name": "input"}, ["combobox"]],
            [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "tel"}], "name": "input"}, ["combobox"]],
            [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "text"}], "name": "input"}, ["combobox"]],
            [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["combobox"]],
            [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["combobox"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "multiple"}, {"constraints": ["undefined"], "name": "size"}], "name": "select"}, ["combobox"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "multiple"}, {"name": "size", "value": 1}], "name": "select"}, ["combobox"]],
            [{"name": "aside"}, ["complementary"]],
            [{"constraints": ["direct descendant of document"], "name": "footer"}, ["contentinfo"]],
            [{"name": "dd"}, ["definition"]],
            [{"name": "dialog"}, ["dialog"]],
            [{"name": "body"}, ["document"]],
            [{"name": "figure"}, ["figure"]],
            [{"attributes": [{"constraints": ["set"], "name": "aria-label"}], "name": "form"}, ["form"]],
            [{"attributes": [{"constraints": ["set"], "name": "aria-labelledby"}], "name": "form"}, ["form"]],
            [{"attributes": [{"constraints": ["set"], "name": "name"}], "name": "form"}, ["form"]],
            [{"name": "span"}, ["generic"]],
            [{"name": "div"}, ["generic"]],
            [{"attributes": [{"name": "role", "value": "grid"}], "name": "table"}, ["grid"]],
            [{"attributes": [{"name": "role", "value": "gridcell"}], "name": "td"}, ["gridcell"]],
            [{"name": "details"}, ["group"]],
            [{"name": "fieldset"}, ["group"]],
            [{"name": "optgroup"}, ["group"]],
            [{"name": "h1"}, ["heading"]],
            [{"name": "h2"}, ["heading"]],
            [{"name": "h3"}, ["heading"]],
            [{"name": "h4"}, ["heading"]],
            [{"name": "h5"}, ["heading"]],
            [{"name": "h6"}, ["heading"]],
            [{"attributes": [{"constraints": ["set"], "name": "alt"}], "name": "img"}, ["img"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "alt"}], "name": "img"}, ["img"]],
            [{"attributes": [{"name": "href"}], "name": "a"}, ["link"]],
            [{"attributes": [{"name": "href"}], "name": "area"}, ["link"]],
            [{"attributes": [{"name": "href"}], "name": "link"}, ["link"]],
            [{"name": "menu"}, ["list"]],
            [{"name": "ol"}, ["list"]],
            [{"name": "ul"}, ["list"]],
            [{"attributes": [{"constraints": [">1"], "name": "size"}, {"name": "multiple"}], "name": "select"}, ["listbox"]],
            [{"attributes": [{"constraints": [">1"], "name": "size"}], "name": "select"}, ["listbox"]],
            [{"attributes": [{"name": "multiple"}], "name": "select"}, ["listbox"]],
            [{"name": "datalist"}, ["listbox"]],
            [{"constraints": ["direct descendant of ol, ul or menu"], "name": "li"}, ["listitem"]],
            [{"name": "main"}, ["main"]],
            [{"name": "math"}, ["math"]],
            [{"name": "menuitem"}, ["command", "menuitem"]],
            [{"name": "nav"}, ["navigation"]],
            [{"name": "option"}, ["option"]],
            [{"name": "progress"}, ["progressbar"]],
            [{"attributes": [{"name": "type", "value": "radio"}], "name": "input"}, ["radio"]],
            [{"attributes": [{"constraints": ["set"], "name": "aria-label"}], "name": "section"}, ["region"]],
            [{"attributes": [{"constraints": ["set"], "name": "aria-labelledby"}], "name": "section"}, ["region"]],
            [{"name": "frame"}, ["region"]],
            [{"name": "tr"}, ["row"]],
            [{"name": "tbody"}, ["rowgroup"]],
            [{"name": "tfoot"}, ["rowgroup"]],
            [{"name": "thead"}, ["rowgroup"]],
            [{"attributes": [{"name": "scope", "value": "row"}], "name": "th"}, ["rowheader"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "search"}], "name": "input"}, ["searchbox"]],
            [{"name": "hr"}, ["separator"]],
            [{"attributes": [{"name": "type", "value": "range"}], "name": "input"}, ["slider"]],
            [{"attributes": [{"name": "type", "value": "number"}], "name": "input"}, ["spinbutton"]],
            [{"name": "output"}, ["status"]],
            [{"name": "table"}, ["table"]],
            [{"name": "dfn"}, ["term"]],
            [{"name": "dt"}, ["term"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "type"}, {"constraints": ["undefined"], "name": "list"}], "name": "input"}, ["textbox"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "email"}], "name": "input"}, ["textbox"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "tel"}], "name": "input"}, ["textbox"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "text"}], "name": "input"}, ["textbox"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["textbox"]],
            [{"name": "textarea"}, ["textbox"]]
          ]
        )
      );
      expect([...elementRoleMap]).toEqual(
        expect.arrayContaining(
          [
            [{"name": "menuitem"}, ["command", "menuitem"]],
            [{"name": "rel"}, ["roletype"]],
            [{"name": "article"}, ["article"]],
            [{"constraints": ["direct descendant of document"], "name": "header"}, ["banner"]],
            [{"attributes": [{"constraints": ["set"], "name": "aria-pressed"}, {"name": "type", "value": "checkbox"}], "name": "input"}, ["button"]],
            [{"attributes": [{"name": "aria-expanded", "value": "false"}], "name": "summary"}, ["button"]],
            [{"attributes": [{"name": "aria-expanded", "value": "true"}], "constraints": ["direct descendant of details element with the open attribute defined"], "name": "summary"}, ["button"]],
            [{"attributes": [{"name": "type", "value": "button"}], "name": "input"}, ["button"]],
            [{"attributes": [{"name": "type", "value": "image"}], "name": "input"}, ["button"]],
            [{"attributes": [{"name": "type", "value": "reset"}], "name": "input"}, ["button"]],
            [{"attributes": [{"name": "type", "value": "submit"}], "name": "input"}, ["button"]],
            [{"name": "button"}, ["button"]],
            [{"constraints": ["descendant of table"], "name": "td"}, ["cell"]],
            [{"attributes": [{"name": "type", "value": "checkbox"}], "name": "input"}, ["checkbox"]],
            [{"name": "th"}, ["columnheader"]],
            [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "email"}], "name": "input"}, ["combobox"]],
            [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "search"}], "name": "input"}, ["combobox"]],
            [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "tel"}], "name": "input"}, ["combobox"]],
            [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "text"}], "name": "input"}, ["combobox"]],
            [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["combobox"]],
            [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["combobox"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "multiple"}, {"constraints": ["undefined"], "name": "size"}], "name": "select"}, ["combobox"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "multiple"}, {"name": "size", "value": 1}], "name": "select"}, ["combobox"]],
            [{"name": "aside"}, ["complementary"]],
            [{"constraints": ["direct descendant of document"], "name": "footer"}, ["contentinfo"]],
            [{"name": "dd"}, ["definition"]],
            [{"name": "dialog"}, ["dialog"]],
            [{"name": "body"}, ["document"]],
            [{"name": "figure"}, ["figure"]],
            [{"attributes": [{"constraints": ["set"], "name": "aria-label"}], "name": "form"}, ["form"]],
            [{"attributes": [{"constraints": ["set"], "name": "aria-labelledby"}], "name": "form"}, ["form"]],
            [{"attributes": [{"constraints": ["set"], "name": "name"}], "name": "form"}, ["form"]],
            [{"name": "span"}, ["generic"]],
            [{"name": "div"}, ["generic"]],
            [{"attributes": [{"name": "role", "value": "grid"}], "name": "table"}, ["grid"]],
            [{"attributes": [{"name": "role", "value": "gridcell"}], "name": "td"}, ["gridcell"]],
            [{"name": "details"}, ["group"]],
            [{"name": "fieldset"}, ["group"]],
            [{"name": "optgroup"}, ["group"]],
            [{"name": "h1"}, ["heading"]],
            [{"name": "h2"}, ["heading"]],
            [{"name": "h3"}, ["heading"]],
            [{"name": "h4"}, ["heading"]],
            [{"name": "h5"}, ["heading"]],
            [{"name": "h6"}, ["heading"]],
            [{"attributes": [{"constraints": ["set"], "name": "alt"}], "name": "img"}, ["img"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "alt"}], "name": "img"}, ["img"]],
            [{"attributes": [{"name": "href"}], "name": "a"}, ["link"]],
            [{"attributes": [{"name": "href"}], "name": "area"}, ["link"]],
            [{"attributes": [{"name": "href"}], "name": "link"}, ["link"]],
            [{"name": "menu"}, ["list"]],
            [{"name": "ol"}, ["list"]],
            [{"name": "ul"}, ["list"]],
            [{"attributes": [{"constraints": [">1"], "name": "size"}, {"name": "multiple"}], "name": "select"}, ["listbox"]],
            [{"attributes": [{"constraints": [">1"], "name": "size"}], "name": "select"}, ["listbox"]],
            [{"attributes": [{"name": "multiple"}], "name": "select"}, ["listbox"]],
            [{"name": "datalist"}, ["listbox"]],
            [{"constraints": ["direct descendant of ol, ul or menu"], "name": "li"}, ["listitem"]],
            [{"name": "main"}, ["main"]],
            [{"name": "math"}, ["math"]],
            [{"name": "menuitem"}, ["command", "menuitem"]],
            [{"name": "nav"}, ["navigation"]],
            [{"name": "option"}, ["option"]],
            [{"name": "progress"}, ["progressbar"]],
            [{"attributes": [{"name": "type", "value": "radio"}], "name": "input"}, ["radio"]],
            [{"attributes": [{"constraints": ["set"], "name": "aria-label"}], "name": "section"}, ["region"]],
            [{"attributes": [{"constraints": ["set"], "name": "aria-labelledby"}], "name": "section"}, ["region"]],
            [{"name": "frame"}, ["region"]],
            [{"name": "tr"}, ["row"]],
            [{"name": "tbody"}, ["rowgroup"]],
            [{"name": "tfoot"}, ["rowgroup"]],
            [{"name": "thead"}, ["rowgroup"]],
            [{"attributes": [{"name": "scope", "value": "row"}], "name": "th"}, ["rowheader"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "search"}], "name": "input"}, ["searchbox"]],
            [{"name": "hr"}, ["separator"]],
            [{"attributes": [{"name": "type", "value": "range"}], "name": "input"}, ["slider"]],
            [{"attributes": [{"name": "type", "value": "number"}], "name": "input"}, ["spinbutton"]],
            [{"name": "output"}, ["status"]],
            [{"name": "table"}, ["table"]],
            [{"name": "dfn"}, ["term"]],
            [{"name": "dt"}, ["term"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "type"}, {"constraints": ["undefined"], "name": "list"}], "name": "input"}, ["textbox"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "email"}], "name": "input"}, ["textbox"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "tel"}], "name": "input"}, ["textbox"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "text"}], "name": "input"}, ["textbox"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["textbox"]],
            [{"name": "textarea"}, ["textbox"]]
          ]
        )
      );
    });
    it('should support the for..of pattern', function () {
      const output = [];
      for (const [key, value] of elementRoleMap) {
        output.push([key, value]);
      }
      expect(output).toEqual(
        expect.arrayContaining(
          [
            [{"name": "menuitem"}, ["command", "menuitem"]],
            [{"name": "rel"}, ["roletype"]],
            [{"name": "article"}, ["article"]],
            [{"constraints": ["direct descendant of document"], "name": "header"}, ["banner"]],
            [{"attributes": [{"constraints": ["set"], "name": "aria-pressed"}, {"name": "type", "value": "checkbox"}], "name": "input"}, ["button"]],
            [{"attributes": [{"name": "aria-expanded", "value": "false"}], "name": "summary"}, ["button"]],
            [{"attributes": [{"name": "aria-expanded", "value": "true"}], "constraints": ["direct descendant of details element with the open attribute defined"], "name": "summary"}, ["button"]],
            [{"attributes": [{"name": "type", "value": "button"}], "name": "input"}, ["button"]],
            [{"attributes": [{"name": "type", "value": "image"}], "name": "input"}, ["button"]],
            [{"attributes": [{"name": "type", "value": "reset"}], "name": "input"}, ["button"]],
            [{"attributes": [{"name": "type", "value": "submit"}], "name": "input"}, ["button"]],
            [{"name": "button"}, ["button"]],
            [{"constraints": ["descendant of table"], "name": "td"}, ["cell"]],
            [{"attributes": [{"name": "type", "value": "checkbox"}], "name": "input"}, ["checkbox"]],
            [{"name": "th"}, ["columnheader"]],
            [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "email"}], "name": "input"}, ["combobox"]],
            [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "search"}], "name": "input"}, ["combobox"]],
            [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "tel"}], "name": "input"}, ["combobox"]],
            [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "text"}], "name": "input"}, ["combobox"]],
            [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["combobox"]],
            [{"attributes": [{"constraints": ["set"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["combobox"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "multiple"}, {"constraints": ["undefined"], "name": "size"}], "name": "select"}, ["combobox"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "multiple"}, {"name": "size", "value": 1}], "name": "select"}, ["combobox"]],
            [{"name": "aside"}, ["complementary"]],
            [{"constraints": ["direct descendant of document"], "name": "footer"}, ["contentinfo"]],
            [{"name": "dd"}, ["definition"]],
            [{"name": "dialog"}, ["dialog"]],
            [{"name": "body"}, ["document"]],
            [{"name": "figure"}, ["figure"]],
            [{"attributes": [{"constraints": ["set"], "name": "aria-label"}], "name": "form"}, ["form"]],
            [{"attributes": [{"constraints": ["set"], "name": "aria-labelledby"}], "name": "form"}, ["form"]],
            [{"attributes": [{"constraints": ["set"], "name": "name"}], "name": "form"}, ["form"]],
            [{"name": "span"}, ["generic"]],
            [{"name": "div"}, ["generic"]],
            [{"attributes": [{"name": "role", "value": "grid"}], "name": "table"}, ["grid"]],
            [{"attributes": [{"name": "role", "value": "gridcell"}], "name": "td"}, ["gridcell"]],
            [{"name": "details"}, ["group"]],
            [{"name": "fieldset"}, ["group"]],
            [{"name": "optgroup"}, ["group"]],
            [{"name": "h1"}, ["heading"]],
            [{"name": "h2"}, ["heading"]],
            [{"name": "h3"}, ["heading"]],
            [{"name": "h4"}, ["heading"]],
            [{"name": "h5"}, ["heading"]],
            [{"name": "h6"}, ["heading"]],
            [{"attributes": [{"constraints": ["set"], "name": "alt"}], "name": "img"}, ["img"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "alt"}], "name": "img"}, ["img"]],
            [{"attributes": [{"name": "href"}], "name": "a"}, ["link"]],
            [{"attributes": [{"name": "href"}], "name": "area"}, ["link"]],
            [{"attributes": [{"name": "href"}], "name": "link"}, ["link"]],
            [{"name": "menu"}, ["list"]],
            [{"name": "ol"}, ["list"]],
            [{"name": "ul"}, ["list"]],
            [{"attributes": [{"constraints": [">1"], "name": "size"}, {"name": "multiple"}], "name": "select"}, ["listbox"]],
            [{"attributes": [{"constraints": [">1"], "name": "size"}], "name": "select"}, ["listbox"]],
            [{"attributes": [{"name": "multiple"}], "name": "select"}, ["listbox"]],
            [{"name": "datalist"}, ["listbox"]],
            [{"constraints": ["direct descendant of ol, ul or menu"], "name": "li"}, ["listitem"]],
            [{"name": "main"}, ["main"]],
            [{"name": "math"}, ["math"]],
            [{"name": "menuitem"}, ["command", "menuitem"]],
            [{"name": "nav"}, ["navigation"]],
            [{"name": "option"}, ["option"]],
            [{"name": "progress"}, ["progressbar"]],
            [{"attributes": [{"name": "type", "value": "radio"}], "name": "input"}, ["radio"]],
            [{"attributes": [{"constraints": ["set"], "name": "aria-label"}], "name": "section"}, ["region"]],
            [{"attributes": [{"constraints": ["set"], "name": "aria-labelledby"}], "name": "section"}, ["region"]],
            [{"name": "frame"}, ["region"]],
            [{"name": "tr"}, ["row"]],
            [{"name": "tbody"}, ["rowgroup"]],
            [{"name": "tfoot"}, ["rowgroup"]],
            [{"name": "thead"}, ["rowgroup"]],
            [{"attributes": [{"name": "scope", "value": "row"}], "name": "th"}, ["rowheader"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "search"}], "name": "input"}, ["searchbox"]],
            [{"name": "hr"}, ["separator"]],
            [{"attributes": [{"name": "type", "value": "range"}], "name": "input"}, ["slider"]],
            [{"attributes": [{"name": "type", "value": "number"}], "name": "input"}, ["spinbutton"]],
            [{"name": "output"}, ["status"]],
            [{"name": "table"}, ["table"]],
            [{"name": "dfn"}, ["term"]],
            [{"name": "dt"}, ["term"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "type"}, {"constraints": ["undefined"], "name": "list"}], "name": "input"}, ["textbox"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "email"}], "name": "input"}, ["textbox"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "tel"}], "name": "input"}, ["textbox"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "text"}], "name": "input"}, ["textbox"]],
            [{"attributes": [{"constraints": ["undefined"], "name": "list"}, {"name": "type", "value": "url"}], "name": "input"}, ["textbox"]],
            [{"name": "textarea"}, ["textbox"]]
          ]
        )
      );
    });
  });
});
